nodeGen <- function(scores, models=NULL, map=NULL, leaf=FALSE,
                    minSimilarity=0.80, eset=NULL,
                    minClusterSize=4, alfa=0.05, verbose=interactive(),...) {


  ## should the node be a leaf? in which case the 'scores' arent really
  ## modelScores but a list of members (sampleNames)
  if(leaf) {
    if(verbose) {
      cat("Generating a leaf of these samples:\n")
      print(scores)
    }
    result <- new("node")
    result@children <- 0
    result@members <- list(scores)
    result@type <- "leaf"
    return(result)
  }

  ## these are the samples coming in to the node
  samples <- rownames(models[1]@scores)

  ## dont even try to split the data if it has less than stopSize elements
  if(length(samples) < 2 * minClusterSize) 
    return(nodeGen(samples, leaf=TRUE))

  ## Filter away all 'insignificant' splits
  bigEnough <- apply(scores@clusterings, 2,
                     function(x) {
                       min(table(x)) >= minClusterSize
                     })

  if(any(bigEnough)) 
    pvals <- lazyModelBootstrap(models, scores,...)
  
  scoreFilter <- mapply(all, pvals$corr <= alfa, bigEnough)

  if(!any(scoreFilter))
    return(nodeGen(samples, leaf=TRUE))

  if(verbose)
    cat("Found ", sum(scoreFilter), " interesting gene class(es)\n")

  scores <- scores[scoreFilter]

  winner <- names(sort(-scores@scores)[1])
  i <- 2
  ba <- try(cv <- mean(nodeCv(models[winner], scores[winner], eset)))
  if(inherits(ba, "try-error")) {
    debug(nodeGen)
    nodeGen(scores, models, map, leaf, minSimilarity, eset, minClusterSize, alfa, verbose, K=50)
  }
  while(cv < 0.9) {
    ## out of models to perform x-val on
    if(i > length(scores@scores))
       return(nodeGen(samples, leaf=TRUE))
    winner <- names(sort(-scores@scores)[i])
    i <- i + 1
    cv <- mean(nodeCv(models[winner], scores[winner], eset))
  }
  grouping <- winner
  if(sum(scoreFilter) > 1) {
    daOrder <- order(-scores@scores)
    for(i in 1:length(scores@scores)) {
      if(mean(c(compareMatchedClasses(scores@clusterings[,winner], scores@clusterings[,daOrder[i]])$crand))
         >= minSimilarity) {
        grouping <- c(grouping, names(scores@scores[daOrder[i]]))
      }
      grouping <- unique(grouping)
    }
  }

  if(length(grouping) > 1) {
    joined <- joinClusterings(scores[grouping])
    node <- new("node",
                ranks=match(grouping, names(sort(-scores@scores))),
                classes=grouping,
                clustering=joined$clustering,
                children=length(unique(joined$clustering)),
                scores=joined$scores@scores,
                members=split(samples, joined$clustering),
                sizes=scores@sizes[grouping],
                type="node")
  }
  else 
    node <- new("node",
                ranks=match(grouping, names(sort(-scores@scores))),
                classes=grouping,
                clustering=scores@clusterings[,grouping],
                children=length(unique(scores@clusterings[,grouping])),
                scores=scores@scores[grouping],
                members=split(samples, scores@clusterings[,grouping]),
                sizes=scores@sizes[grouping],
                type="node")

  node
}

joinClusterings <- function(scores) {
  ## what...combines many clusterings to one by simple majority vote
  ## taking the uncertainty of the clusterings into account
  ## scores...a modelScores object
  cert <- split(scores@certainty, 1:nrow(scores@certainty))
  cl <- split(scores@clusterings, 1:nrow(scores@clusterings))
  clustering <- mapply(function(cl, cert)
                       {
                         sp <- split(cert, cl)
                         votes <- unlist(lapply(sp, sum))
                         winner <- which(votes == max(votes))
                         if(length(winner) != 1)
                           winner <- sample(winner, 1)
                         as.integer(names(winner))
                       }, cl, cert)
  list(clustering=clustering, scores=scores)
}


nodeCv <- function(model, score, eset, N=10, K=10) {

  ## what...Computes LOOCV on a node. This is done by computing a new PCA  
  ## leaving out one sample at a time, projecting down the left out sample 
  ## on the first PC and predicting its cluster label by minimizing the    
  ## squared sum on the medoids obtained when clustering all samples. The  
  ## result is given as a vector with the amount of misclassified samples  
  ## belonging to the corresponding cluster. This can be conveniently      
  ## exploited when constructing the trees by simple adding a number to    
  ## corresponding edge.                                                   
  ## x...a nipalsPca object                                                
  ## masterClu...the clustering to test, (or null)                         
  ## scale...indicates whether the data should be scaled prior to PCA      
  ## center...see above                                                    
  ## when...2005-03-07                                                     

  size <- model@nobs %/% N
  if(class(eset) == "exprSet")
    eset <- exprs(eset)

  res <- NULL
  for(j in 1:K){
    cvSubsets <- NULL
    dseq <- 1:model@nobs
    for(i in 1:N) {
      cvSubsets <- rbind(cvSubsets, sample(dseq, size))
      dseq <- dseq[-match(cvSubsets[i,], dseq)]
    }

    for(i in 1:nrow(cvSubsets)) {
      x <- eset[model@subset, -cvSubsets[i,]]
      tempModel <- pca(t(x), center=model@centered, scale=model@scaled,
                       varLimit=model@varLimit)
      tempScore <- clusterGeneClass(tempModel, scoretype=score@scoretype)$clustering
      tempMast <- score@clusterings[-cvSubsets[i,]]
      tmp <- try(res <- c(res, mean(c(compareMatchedClasses(tempMast, tempScore)$crand))))
      if(class(tmp) == "try-error")
        cat("tempMast\n", tempMast, "tempScore\n", tempScore, "\n")
    }

  }
  res
}
