\name{pca}
\alias{pca}
\title{Perform principal component analysis}
\description{Can be used for computing PCA on a numeric
  matrix using either the standard \code{prcomp} method or NIPALS
  (Nonlinear Iterative Partial Least Squares)
  algorithm which is an iterative approach for estimating the principal
  components extracting them one at a time.}
\usage{pca(object, k=2, scale=c("none", "pareto", "vector", "UV"), center=TRUE,
limit=1e-6, maxiterations=5000, varLimit=1,
subset=numeric(), method=c("svd", "nipals"), verbose=FALSE, ...)}
\arguments{
  \item{object}{Numerical matrix  with (or on object convertable to such) with
    samples in rows and variables as columns. Also takes exprSet in
    which case the transposed exprs slot is used.}
  \item{k}{Number of components that should be extracted.}
  \item{center}{Indicates if the matrix should be mean centered or not.}
  \item{limit}{The limit condition for judging if the algorthim has
    converged or not, specifically if a new iteration is done if
    \eqn{(T_{old} - T)^T(T_{old} - T) > \code{limit}}.}
  \item{maxiterations}{Defines how many iterations can be done before the
    algorithm should abort (happens almost exlusively when there were
    some wrong in the input data).}
  \item{verbose}{If TRUE a simple progress bar is displayed}
  \item{varLimit}{If defined, \code{k} is ignored and the algorithm
    continues until the explained variance is greater than this number
    (<1).}
  \item{scale}{One of "UV" (unit variance \eqn{a=a/\sigma_{a}})
    "vector" (vector normalization \eqn{b=b/||b||}), "pareto" or "none"
    to indicate which
    scaling should be used to scale the matrix with \eqn{a} variables
    and \eqn{b} samples.}
  \item{subset}{For convenience one can pass a large matrix but only
    use the variable specified as subset. Can be colnames or indices.}
  \item{method}{One of "svd" or "nipals". "svd" makes \code{pca} use
    \code{prcomp}}
  \item{...}{Pass through arguments}
}
\details{
  NIPALS is capable of handling missing values (by simply leaving them
  out of the dot-product) provided they are well scattered in the matrix
  and relatively few. The NIPALS and SVD should (except for rounding
  errors) yield identical result but SVD is much faster and should be
  used as default. \code{pca} therefore does not add much to
  \code{prcomp} except a maybe more convenient way of representing the result.
}

\value{
  A \code{pcaRes} object.
}
\references{
  Wold, H. (1966) Estimation of principal components and related models by
  iterative least squares. In Multivariate Analysis (Ed.,
  P.R. Krishnaiah), Academic Press, NY, 391-420.
}
\author{Henning Redestig}
\seealso{\code{prcomp}, \code{princomp}}
\examples{
data(iris)
## Usually some kind of scaling is appropriate
pcIr <- pca(iris[,1:4], scale="UV", method="nipals")
pcIr <- pca(iris[,1:4], scale="UV", method="svd")
## Get a short summary on the calculated model
summary(pcIr)
## Scores and loadings plot
slplot(pcIr, sl=as.character(iris[,5]))
}
\keyword{multivariate}
